//Servo library toevoegen
#include <Servo.h>

//Servo defineren
Servo servoLinkerbeen;
Servo servoRechterbeen;
Servo servoHeup;
Servo servoHoofd;

//positie defineren
int legPositie = 0;
int legCommand = 3;
bool fire = false;
int firedelay = 0;
bool fireOn;

const int minHoek = 1000;
const int maxHoek = 2000;
const int moveSlow = 100;
const int moveFast = 200;

const int initDelay = 300;

void setup()
{
  Serial.begin(9600);
  //geeft aan op welke pin de servos en LEDs aangesloten zijn
  servoLinkerbeen.attach(5);
  servoRechterbeen.attach(6);
  servoHeup.attach(9);
  servoHoofd.attach(10);
  pinMode(11, OUTPUT);

  initTest(); //Runs a test to see if all components are still working
}

void initTest() {
  servoLinkerbeen.write(minHoek);
  delay(initDelay);
  servoLinkerbeen.write(maxHoek);
  delay(initDelay);
  servoLinkerbeen.write((minHoek + maxHoek) / 2);
  delay(initDelay);

  servoRechterbeen.write(minHoek);
  delay(initDelay);
  servoRechterbeen.write(maxHoek);
  delay(initDelay);
  servoRechterbeen.write((minHoek + maxHoek) / 2);
  delay(initDelay);

  servoHeup.write(minHoek);
  delay(initDelay);
  servoHeup.write(maxHoek);
  delay(initDelay);
  servoHeup.write((minHoek + maxHoek) / 2);
  delay(initDelay);

  servoHoofd.write(minHoek+(maxHoek-minHoek)/3);
  delay(initDelay);
  servoHoofd.write(maxHoek -(maxHoek-minHoek)/3);
  delay(initDelay);
  servoHoofd.write((minHoek + maxHoek) / 2);
  delay(initDelay);

  for (int i = 0; i < 5; i++) {
    ShootOn();
    delay(initDelay / 2);
    ShootOff();
    delay(initDelay / 2);
  }
}

void loop()
{
  if (Serial.available()) //if the serial connection is on
  {
    int serialCommand = Serial.read(); //read the first byte coming in
    int serialParameter;
    int serialEnding;
    bool serialRead = false;
    if (serialCommand > 0 && serialCommand <= 5) {
      serialParameter = Serial.read(); //read the second byte coming in
      if (serialParameter > 0 && serialParameter <= 255) {
        serialEnding = Serial.read(); //read the ending byte to assemble the complete command
        if (serialEnding == 0) {
          serialRead = true;
        }
      }
    }

    if (serialRead) {
      switch (serialCommand)//run the command sent from Unity
      {
        case 1:
          fire = true;
          fireOn = false;
          firedelay = 1;
          break;
        case 2:
          fire = false;
          break;
        case 3:
          TurnBody(serialParameter);
          break;
        case 4:
          TurnHead(serialParameter);
          break;
        case 5:
          legCommand = serialParameter;
          break;
      }
    }
  }
  MoveLegs();
  Shooting();
  delay(50);
}

void MoveLegs() {
  int old = legPositie;
  switch (legCommand)
  {
    case 1: // hard achter
      legPositie -= moveFast;
      break;
    case 2: // achter
      legPositie -= moveSlow;
      break;
    case 3: // stil
      legPositie = (minHoek + maxHoek) / 2;
      break;
    case 4: // voor
      legPositie += moveSlow;
      break;
    case 5: // hard voor
      legPositie += moveFast;
      break;
    default:
      legPositie = (minHoek + maxHoek) / 2;
      break;
  }
  
  if (legPositie > maxHoek) legPositie = minHoek;
  if (legPositie < minHoek) legPositie = maxHoek;
  
  if (old != legPositie) {
    servoLinkerbeen.write(legPositie); // 1000 => 2000; 2000 => 1000
    delay(50);//this delay is required because the arduino UNO can't move two servos in one frame
    servoRechterbeen.write(legPositie);
  }
}

void Shooting() {
  if (fire) {
    firedelay++;
    if (firedelay > 1) {
      firedelay = 0;
      fireOn = !fireOn;
      if (fireOn) {
        ShootOn();
      } else {
        ShootOff();
      }
    }
  } else {
    ShootOff();
  }
}

void ShootOn() {
  digitalWrite(11, HIGH);
}

void ShootOff() {
  digitalWrite(11, LOW);
}

void TurnBody(int angle) { // 1-90, this calculation turns the servo from 90 to 1 degrees based on the input from Unity
  servoHeup.write(minHoek + ((maxHoek - minHoek) / 90)*(90-angle));
}

void TurnHead(int angle) { // 1-90, this calculation turns the servo from 1 to 90 degrees based on the input from Unity
  servoHoofd.write(minHoek + ((maxHoek - minHoek) / 90)*angle);
}
